#!BPY

"""
Name: 'Python (.py)...'
Blender: 237
Group: 'Export'
Tooltip: 'Generate a python script for each selected mesh'
"""
__author__ = " Jean-Baptiste PERIN (jb_perin(at)yahoo.fr)"
__url__ = ("blender", "elysiun",
"PYTHON EXPORTER , http://www.zoo-logique.org/3D.Blender/index.php3?zoo=dld&rep=zip ",
"Communicate problems and errors, http://www.zoo-logique.org/3D.Blender/newsportal/thread.php?group=3D.Blender") 

__version__ = "0.2"

__bpydoc__ = """ python_export.py v0.2

Script generating a python script which build selected meshes,  
Usage:<br>
 - Select meshes you want to export <br>
 - Launch this script (Alt-P);<br>
A python script is generated that build these meshes.
Generated scripts are stored into <blender_exe_path>/.blender/scripts directory<br>
For them to be reachable through your script menu:<br>
 - Open a Script window and perform a Script->Update Menus <br>
Generated script can then be launched from menu Script->Add->your_mesh_name<br>
Python output takes into account: <br>
- vertices coordinates, uv coordinates <br>
- object position (relatively to 3D cursor), size and rotation <br>
- object and mesh name <br>
"""

#----------------------------------------------
# (c) Jean-Baptiste PERIN  octobre 2004, released under Blender Artistic Licence
#    for the Blender 2.37 Python Scripts Bundle.
#----------------------------------------------

import Blender
from Blender import *

blender_path = Blender.sys.progname
blender_install_dir = sys.dirname(sys.splitext((blender_path))[0])
blender_script_dir = blender_install_dir+sys.sep +'.blender'+sys.sep +'scripts'

def exportPython(filename):

  #
  # List selected meshes
  #
  lesobjets = Blender.Object.GetSelected()
  lesmesh = []
  for ob in lesobjets:
    if ob.getType()=="Mesh":
      lesmesh.append(ob)
  #
  # Build object name (add menu entry) from filename
  #
  objectname =  Blender.sys.splitext(Blender.sys.basename(filename))[0]
  if objectname.find('add') == 0:
    objectname = objectname [3:]

  #
  # Create or ReCreate output python file
  #
  lescript = open (filename,"w")

  #
  # write file header
  #
  lescript.write("#!BPY\n")
  lescript.write("\"\"\"\n")
  lescript.write("Name: '"+objectname+"'\n")
  lescript.write("Blender: 237\n")
  lescript.write("Group: 'Add'\n")
  lescript.write("Tooltip: 'Add a "+objectname+" to current scene'\n")
  lescript.write("\"\"\"\n")

  lescript.write("import Blender\n")
  lescript.write("from Blender import *\n")
  lescript.write("def computeName(radical):\n")
  lescript.write("  nextname=\"\"\n")
  lescript.write("  try:\n")
  lescript.write("    ob = Blender.Object.Get(radical)\n")
  lescript.write("    ii = 1\n")
  lescript.write("    while (ob != None) and (ii<= 999):\n")
  lescript.write("      essai=radical+\".\"+str(ii)\n")
  lescript.write("      try:\n")
  lescript.write("        ob = Blender.Object.Get(essai)\n")
  lescript.write("      except:\n")
  lescript.write("        nextname=essai\n")
  lescript.write("        ob = None\n")
  lescript.write("      ii+=1\n")
  lescript.write("  except:\n")
  lescript.write("    nextname=radical\n")
  lescript.write("  return (nextname)\n")
  lescript.write("\n")
  lescript.write("obname = computeName('"+objectname+"')\n")

  #
  # for each selected mesh
  #
  for ob in lesmesh:
    lamesh = ob.getData ()


    #
    # write python commands that create the mesh
    #
    lescript.write("me=NMesh.New(obname)\n")

    #
    # write python commands that build vertices 
    #
    lesvertices =  lamesh.verts
    for ve in lesvertices:
      lescript.write("v=NMesh.Vert("+str(ve.co[0])+","+str(ve.co[1])+","+str(ve.co[2])+")\n")
      lescript.write("me.verts.append(v)\n")

    #
    # write python commands that build faces 
    #
    lesfaces = lamesh.faces
    for fa in lesfaces:
      lescript.write("f=NMesh.Face()\n")
      for ve in fa.v:
        lescript.write("f.v.append(me.verts["+str(lamesh.verts.index(ve))+"])\n")
      if lamesh.hasFaceUV():
        lescript.write("f.uv=[]\n")
        for uvc in fa.uv:
          lescript.write("f.uv.append("+str(uvc[0])+","+str(uvc[1])+")\n")
      lescript.write("me.faces.append(f)\n")

    #
    # write python commands that write size, position and rotation 
    #
    
    lescript.write("ob=NMesh.PutRaw(me)\n")
    lescript.write("ob.setName(obname)\n")
    lescript.write("ob.setSize(["+str(ob.getSize()[0])+","+str(ob.getSize()[1])+","+str(ob.getSize()[2])+"])\n")
    lescript.write("ob.setLocation(["+str(ob.getLocation()[0]-Blender.Window.GetCursorPos()[0])+"+Blender.Window.GetCursorPos()[0],"+str(ob.getLocation()[1]-Blender.Window.GetCursorPos()[1])+"+Blender.Window.GetCursorPos()[1],"+str(ob.getLocation()[2]-Blender.Window.GetCursorPos()[2])+"+Blender.Window.GetCursorPos()[2]])\n")
    lescript.write("ob.setEuler(["+str(ob.getEuler()[0])+","+str(ob.getEuler()[1])+","+str(ob.getEuler()[2])+"])\n")


  #
  # Close python script file
  #
  lescript.close()


#
# MAIN
#

lesobjets = Blender.Object.GetSelected()
if len(lesobjets) >= 2:
  filename = blender_script_dir+ sys.sep + 'add'+Blender.sys.splitext(Blender.sys.basename(Blender.Get('filename')))[0] +'.py'
elif len(lesobjets) == 1:
  filename = blender_script_dir+ sys.sep + 'add'+Blender.Object.GetSelected()[0].name +'.py'
Blender.Window.FileSelector(exportPython, "Export Python", filename)
